# frozen_string_literal: true

module BrocadeVRouter
  class Configurator
    CONF_PATH = 'rest/conf'.freeze
    SHOW_CONF_CMD = 'configuration'.freeze

    attr_reader :connection, :last_response, :logger
    attr_accessor :raise_on_fail

    def initialize(connection, base_path: nil, logger: nil, raise_on_fail: true)
      @connection = connection
      @base_path = base_path
      @logger = logger
      @raise_on_fail = raise_on_fail
    end

    def base_path
      @base_path ||=
        begin
          handle_request :post, CONF_PATH
          handle_response(connection.post "/#{CONF_PATH}").headers['location']
        end
    end

    def perform(&blk)
      @context = eval 'self', blk.binding
      instance_eval(&blk) if block_given?
    end

    def set(paths = nil, &blk)
      res = build_paths(paths, &blk).map do |p|
        path = "#{base_path}/set/#{p}"
        handle_request :put, path
        handle_response connection.put path
      end

      res.size > 1 ? res : res.first
    end

    def get(cmd = SHOW_CONF_CMD)
      path = "rest/op/show/#{cmd}"
      handle_request :post, path
      resp = handle_response connection.post "/#{path}"

      return unless resp.success?

      path = "#{resp.headers['location']}"
      handle_request :get, path
      resp = handle_response connection.get "/#{path}"

      resp.success? && cmd == SHOW_CONF_CMD ? Configuration.new(resp.body).to_h : resp.body
    end

    def delete(paths = nil, &blk)
      res = build_paths(paths, &blk).map do |p|
        path = "#{base_path}/delete/#{p}"
        handle_request :put, path
        handle_response connection.put path
      end

      res.size > 1 ? res : res.first
    end

    def commit(path = nil)
      req_path = "#{path || base_path}/commit"
      handle_request :post, req_path
      handle_response connection.post req_path
    rescue RequestError => e
      raise unless e.message['No changes to commit']
    end

    def save(path = nil)
      req_path = "#{path || base_path}/save"
      handle_request :post, req_path
      handle_response connection.post req_path
    end

    def close(path = nil)
      handle_request :delete, path || base_path
      handle_response connection.delete(path || base_path)
    end

    def close_all_sessions!(commit: false, save: false)
      sessions.each do |session|
        path = "#{CONF_PATH}/#{session['id']}"
        commit path if commit
        save path if save
        close path
      end
    end

    def sessions
      (MultiJson.load (handle_response connection.get CONF_PATH).body)
        .fetch('session', [])
    end

    def raise_on_fail?
      @raise_on_fail
    end

    private

    def handle_request(method, path)
      return unless logger
      logger.info "> #{method.to_s.upcase} #{connection.url_prefix}#{path}"
    end

    def handle_response(resp)
      @last_response = resp

      message = "< #{resp.env.method.upcase} #{resp.env.url} #{resp.env.status}\n#{resp.body}"

      if resp.success?
        logger.info message if logger
      else
        logger.warn message if logger
        raise RequestError.new message if raise_on_fail?
      end

      resp
    end

    def method_missing(name, value = nil, exec = false, &blk)
      PathBuilder.new(@context) { public_send name, value, exec, &blk }.to_a
    end

    def build_paths(paths, &blk)
      paths = Array(paths)
      return paths unless block_given?
      paths.map { |p| PathBuilder.new(@context, p, &blk).to_a }.flatten
    end
  end
end
